# coding:utf-8
# created by tongshiwei on 2018/10/9


from .indicesor import Indicesor
from .splitter.file_splitter import *
from .splitter.iterable_splitter import *
from .splitter.splitter import Splitter
